@@ -33,7 +33,7 @@
                 EL_TYPE_HOME_APPLET, ELHomeAppletPrivate))
 
 #define BOX_WIDTH 352
-#define BOX_HEIGHT 252
+#define BOX_HEIGHT 266
 
 #define C_WIDTH (BOX_WIDTH - 2*HILDON_MARGIN_HALF)
 #define C_HEIGHT (BOX_HEIGHT - 2*HILDON_MARGIN_HALF)
@@ -54,6 +54,8 @@ struct _ELHomeAppletPrivate
         GtkWidget *message;
         GtkWidget *icon;
         GtkWidget *unread;
+        GtkWidget *received;
+        GtkWidget *empty;
 
         gint       event_id;
 
@@ -62,6 +64,8 @@ struct _ELHomeAppletPrivate
         guint unread_count;
 
         const gchar *current_font;
+
+        guint idle_id;
 };
 
 HD_DEFINE_PLUGIN_MODULE (ELHomeApplet, el_home_applet, HD_TYPE_HOME_PLUGIN_ITEM);
@@ -143,6 +147,7 @@ expose_event (GtkWidget *self, GdkEventExpose *event)
         gdk_cairo_region (cr, event->region);
         cairo_clip (cr);
 
+        /* draw bound box */
         cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
 
         cairo_set_source_rgba (cr, 0.4f, 0.4f, 0.4f, 0.1f);
@@ -158,6 +163,7 @@ expose_event (GtkWidget *self, GdkEventExpose *event)
         cairo_close_path(cr);
         cairo_stroke (cr);
 
+        /* draw header */
         cairo_set_line_width (cr, 1.0f);
 
         cairo_translate (cr, C_X, C_Y);
@@ -176,6 +182,7 @@ expose_event (GtkWidget *self, GdkEventExpose *event)
         cairo_set_source_rgba (cr, red, green, blue, 1.0f);
         cairo_stroke (cr);
 
+        /* draw body */
         cairo_move_to (cr, 0, HEADER_HEIGHT);
         cairo_line_to (cr, 0, C_HEIGHT - BOX_RADIOUS);
         cairo_curve_to (cr, 0, C_HEIGHT, 0, C_HEIGHT, BOX_RADIOUS, C_HEIGHT);
@@ -185,10 +192,24 @@ expose_event (GtkWidget *self, GdkEventExpose *event)
         cairo_line_to (cr, 0, HEADER_HEIGHT);
         cairo_close_path(cr);
 
-        if (priv->active)
-                cairo_set_source_rgba (cr, red, green, blue, 0.8f);
-        else
-                cairo_set_source_rgba (cr, 0.4f, 0.4f, 0.4f, 0.8f);
+        /* draw body filling depending on (in)active state */
+        cairo_pattern_t *grad;
+        grad = cairo_pattern_create_linear(0, HEADER_HEIGHT,
+                                           0, C_HEIGHT);
+
+        if (priv->active){
+                cairo_pattern_add_color_stop_rgba (grad, 0.5f,
+                                                   red, green, blue, 0.8f);
+                cairo_pattern_add_color_stop_rgba (grad, 1.0f,
+                                                   red/2, green/2, blue/2, 0.8f);
+        }
+        else {
+                cairo_pattern_add_color_stop_rgba (grad, 0.5f,
+                                                   0.4f, 0.4f, 0.4f, 0.8f);
+                cairo_pattern_add_color_stop_rgba (grad, 1.0f,
+                                                   0.2f, 0.2f, 0.2f, 0.8f);
+        }
+        cairo_set_source (cr, grad);
         cairo_fill (cr);
 
         /* cairo_set_source_rgba (cr, red, green, blue, 1.0f); */
@@ -201,7 +222,7 @@ expose_event (GtkWidget *self, GdkEventExpose *event)
         /*                    BOX_RADIOUS); */
         /* cairo_close_path(cr); */
         /* cairo_stroke (cr); */
-
+        cairo_pattern_destroy (grad);
         cairo_destroy (cr);
 
         return GTK_WIDGET_CLASS (el_home_applet_parent_class)->expose_event (self, event);
@@ -212,6 +233,10 @@ dispose (GObject *self)
 {
         ELHomeAppletPrivate *priv = EL_HOME_APPLET(self)->priv;
 
+        if (priv->idle_id){
+                g_source_remove (priv->idle_id);
+                priv->idle_id = 0;
+        }
         if (priv->eventlogger){
                 g_object_unref (priv->eventlogger);
                 priv->eventlogger = NULL;
@@ -226,6 +251,30 @@ finalize (GObject *self)
         G_OBJECT_CLASS (el_home_applet_parent_class)->finalize (self);
 }
 
+static gchar*
+format_time (time_t t)
+{
+        static const guint RESULT_SIZE = 32;
+
+        time_t now;
+        struct tm now_tm, t_tm;
+        const gchar *format = "%Y.%m.%d %T";
+        gchar *result = g_malloc0 (RESULT_SIZE);
+
+        now = time (NULL);
+        localtime_r (&now, &now_tm);
+        localtime_r (&t, &t_tm);
+
+        if ((now_tm.tm_year == t_tm.tm_year) &&
+            (now_tm.tm_mon  == t_tm.tm_mon) &&
+            (now_tm.tm_mday == t_tm.tm_mday))
+                format = "%T";
+
+        strftime (result, RESULT_SIZE, format, &t_tm);
+
+        return result;
+}
+
 static void
 show_event (ELHomeApplet *self, RTComElIter *it)
 {
@@ -233,14 +282,18 @@ show_event (ELHomeApplet *self, RTComElIter *it)
 
         gchar *message = NULL;
         gchar *remote = NULL;
+        gchar *received = NULL;
         const gchar *icon_name = NULL;
 
         if (it && rtcom_el_iter_first (it)){
                 rtcom_el_iter_dup_string (it, "free-text", &message);
                 if (message){
                         const gchar *service;
+                        time_t received_t;
 
                         rtcom_el_iter_get_int (it, "id", &priv->event_id);
+                        if (rtcom_el_iter_get_int (it, "start-time", (gint*)&received_t))
+                                received = format_time (received_t);
 
                         if(!rtcom_el_iter_dup_string (it, "remote-name", &remote))
                                 rtcom_el_iter_dup_string (it, "remote-id", &remote);
@@ -255,8 +308,19 @@ show_event (ELHomeApplet *self, RTComElIter *it)
                 priv->event_id = -1;
         }
 
-        gtk_label_set_text (GTK_LABEL (priv->message), message);
+        if (message){
+                if (!GTK_WIDGET_VISIBLE (priv->message))
+                        gtk_widget_show (priv->message);
+                gtk_label_set_text (GTK_LABEL (priv->message), message);
+        }
+        else{
+                gtk_widget_hide (priv->message);
+                gtk_widget_show (priv->empty);
+        }
+
         gtk_label_set_text (GTK_LABEL (priv->sender), remote);
+        gtk_label_set_text (GTK_LABEL (priv->received), received);
+
         if (icon_name){
                 const gchar *current_icon_name;
                 gtk_image_get_icon_name (GTK_IMAGE (priv->icon),
@@ -317,9 +381,13 @@ update_unread_label (ELHomeApplet *self)
         ELHomeAppletPrivate *priv = self->priv;
         gchar *text;
 
-        text = g_strdup_printf ("%d", priv->unread_count);
-        gtk_label_set_text (GTK_LABEL (priv->unread), text);
-        g_free (text);
+        if (priv->unread_count > 0){
+                text = g_strdup_printf ("%d", priv->unread_count);
+                gtk_label_set_text (GTK_LABEL (priv->unread), text);
+                g_free (text);
+        }
+        else
+                gtk_label_set_text (GTK_LABEL (priv->unread), NULL);
 }
 
 static gint
@@ -382,6 +450,31 @@ mark_as_read (ELHomeApplet *self)
         }
 }
 
+static gboolean
+read_new_event (ELHomeApplet *self)
+{
+        ELHomeAppletPrivate *priv = self->priv;
+
+        read_event (self);
+        priv->unread_count = query_unread_events (priv->eventlogger);
+        update_unread_label (self);
+
+        priv->idle_id = 0;
+
+        return FALSE;
+}
+
+static void
+add_new_idle (ELHomeApplet *self)
+{
+        ELHomeAppletPrivate *priv = self->priv;
+
+        if (priv->idle_id)
+                g_source_remove (priv->idle_id);
+        priv->idle_id = g_idle_add ((GSourceFunc)read_new_event,
+                                    self);
+}
+
 static void
 new_event_cb (RTComEl      *backend,
               gint          event_id,
@@ -392,37 +485,7 @@ new_event_cb (RTComEl      *backend,
               const gchar  *service,
               ELHomeApplet *self)
 {
-        ELHomeAppletPrivate *priv = self->priv;
-        RTComElIter *it = NULL;
-
-        it = make_query (priv->eventlogger, event_id);
-        if (it){
-                if (rtcom_el_iter_first (it)){
-                        show_event (self, it);
-                        priv->unread_count++;
-                        update_unread_label (self);
-                }
-                g_object_unref (it);
-        }
-}
-
-static void
-event_updated_cb (RTComEl      *backend,
-                  gint          event_id,
-                  const gchar  *local_uid,
-                  const gchar  *remote_uid,
-                  const gchar  *remote_ebook_uid,
-                  const gchar  *group_uid,
-                  const gchar  *service,
-                  ELHomeApplet *self)
-{
-        ELHomeAppletPrivate *priv = self->priv;
-
-        if (event_id == priv->event_id)
-                read_event (self);
-
-        priv->unread_count = query_unread_events (priv->eventlogger);
-        update_unread_label (self);
+        add_new_idle (self);
 }
 
 static gboolean
@@ -526,6 +589,21 @@ el_home_applet_init (ELHomeApplet *self)
                                      MESSAGE_HEIGHT);
         gtk_widget_set_name (priv->message, "hildon-shadow-label");
 
+        /* TODO: l10n */
+        priv->empty = gtk_label_new ("No new messages");
+        gtk_widget_set_name (priv->empty, "hildon-shadow-label");
+        GTK_WIDGET_SET_FLAGS (priv->empty, GTK_NO_SHOW_ALL);
+
+        priv->received = gtk_label_new ("aewf aewf aewf awef");
+        gtk_misc_set_alignment (GTK_MISC (priv->received),
+                                1.0f,
+                                0.5f);
+        gtk_widget_set_size_request (priv->received,
+                                     MESSAGE_WIDTH,
+                                     -1);
+        hildon_helper_set_logical_font (priv->received, "SmallSystemFont");
+        gtk_widget_set_name (priv->received, "hildon-shadow-label");
+
         hbox = gtk_hbox_new (FALSE, 0);
         gtk_box_pack_start (GTK_BOX (hbox), priv->unread, FALSE, FALSE, 0);
         gtk_box_pack_start (GTK_BOX (hbox), priv->icon, FALSE, FALSE, 0);
@@ -534,6 +612,8 @@ el_home_applet_init (ELHomeApplet *self)
         vbox = gtk_vbox_new (FALSE, 0);
         gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
         gtk_box_pack_start (GTK_BOX (vbox), priv->message, TRUE, TRUE, 0);
+        gtk_box_pack_start (GTK_BOX (vbox), priv->empty, TRUE, TRUE, 0);
+        gtk_box_pack_start (GTK_BOX (vbox), priv->received, FALSE, FALSE, 0);
 
         align = gtk_alignment_new (0.5f, 0.0f, 1.0f, 1.0f);
         gtk_alignment_set_padding (GTK_ALIGNMENT (align),
@@ -566,12 +646,10 @@ el_home_applet_init (ELHomeApplet *self)
                           self);
         g_signal_connect (priv->eventlogger,
                           "event-updated",
-                          G_CALLBACK (event_updated_cb),
+                          G_CALLBACK (new_event_cb),
                           self);
 
-        read_event (self);
-        priv->unread_count = query_unread_events (priv->eventlogger);
-        update_unread_label (self);
+        read_new_event (self);
 #endif
 }
 
